#!/bin/bash

ARCH=`uname -m`
Arch=`echo $ARCH | sed -e s/i.86/x86/ -e s/x86_64/x86/ -e s/amd64/x86/ -e s/aarch64.*/arm64/ -e s/mips64el/mips/`
TOPDIR=`pwd`/rpmbuild

ARG1="$1"
SUBV=`cat ${TOPDIR}/SPECS/kernel.spec| grep subv | head -n 1 | awk -F' ' '{print $3}'`

trap "cleanup_exit" 2

function cleanup()
{
	sleep 1
	if [ "$ARG1" == "biv" ]; then
		sed -i "s/\%define subv.*/\%define subv $SUBV/" $TOPDIR/SPECS/kernel.spec
	fi
}

function cleanup_exit()
{
	cleanup
	exit -1;
}

git clean -df
rm -rf rpmbuild/BUILD
rm -rf rpmbuild/BUILDROOT

rm -rf rpmbuild/kernel
mkdir rpmbuild/kernel
make mrproper -j 16

rsync -av --exclude rpmbuild --exclude .git . rpmbuild/kernel/

#prepare kernel.tar.gz
cd rpmbuild

if [ "$ARG1" == "biv" ]; then
echo "I'm building an optimized kernel, but it will turn off some options, please note."

echo "CONFIG_FTRACE=n" >> kernel/arch/$Arch/configs/kylinos_generic_defconfig

sed -i "s/\%define subv.*/\%define subv $SUBV.biv/" SPECS/kernel.spec
fi

tar -zcvf kernel.tar.gz kernel
cd ../
rm -rf rpmbuild/SOURCES/kernel.tar.gz
mv rpmbuild/kernel.tar.gz rpmbuild/SOURCES/kernel.tar.gz

#do building
rpmbuild -bb rpmbuild/SPECS/kernel.spec --define="_topdir $TOPDIR"

#clean work
cleanup

if [ $? -eq 0 ]; then
	echo "building rpm success..."
	rm -rf rpmbuild/kernel
else
	echo "building failed..."
	exit
fi

#archive RPMS
RPMDIR=${TOPDIR}/RPMS/${ARCH}
KVRELEASE=4.19.90-kernel-${ARCH}
rm -rf rpmbuild/$KVRELEASE*
mkdir rpmbuild/$KVRELEASE

for pack in `ls $RPMDIR/${rpmpkg}*.rpm | grep -v source`
do
	cp $pack rpmbuild/$KVRELEASE
done

#clean
cd rpmbuild
tar -zcvf ${KVRELEASE}.tar.gz $KVRELEASE
rm -rf $KVRELEASE
echo -e "Archive all builded rpm packages in: \nrpmbuild/${KVRELEASE}.tar.gz"
cd ..
